#
# Provides functionality for analyzing BPMN-graphs
#
require 'rexml/document' 
include REXML

class Array
	def deep_clone
		collect do |n|
			case n
			when Array
				n.deep_clone
			else
				n
			end
		end
	end
end

class BPMNAnalyzer

	attr_accessor :matrix, :tcMatrix, :file, :id, :nodeIds
	
	def initialize(xml,id)
				
		if xml.root.name != "model" 
    		raise "BPMNANALYZER: Cannot process file, root element is not <model>!"
	  	end
	  	
	  	#puts "BPMNANALYZER LOADING " + file + "..."
 	  	
 	  	@nodeIds = Array.new	
 	  	@matrix = Array.new
 	 
 	  	# Process all nodes
	  	xml.elements.each("model/process[@id=#{id}]/node") do |n|	  		
	  		# Add node to array
	  		@nodeIds << n.attributes['id']
	  		end
	  		
	  	# Initialize matrix with 9999 values
	  	@nodeIds.size.times do |y|
	  		@matrix << Array.new
	  		@nodeIds.size.times do |x|
	  				@matrix[y] << 9999
	  			end
	  		end
	  	
	  	# Add directed edges
	  	xml.elements.each("model/process[@id=#{id}]/flow") do |n|
	  		from = n.attributes['from']
	  		to = n.attributes['to']
	  		@matrix[@nodeIds.index(from)][@nodeIds.index(to)] = 1
	  		end	  	
	  		
	  	# Add directed edges between scope events and corresponding activities
	  	xml.elements.each("model/process[@id=#{id}]/node[contains(@type,'Intermediate Event')]") do |n|
	  		scope = n.attributes['scope']
	  		to = n.attributes['id']
	  		if scope
	  			@matrix[@nodeIds.index(scope)][@nodeIds.index(to)] = 1
	  			end
	  		end
	  		
	  	# Calculate transitive closure
	  	@tcMatrix = calculateTransitiveClosure
	  	
	end
	
	def calculateTransitiveClosure(m = @matrix)
		tc = m.deep_clone
		tc.size.times do |k|
			tc.size.times do |i|
				tc.size.times do |j|
					if (tc[i][j] > (tc[i][k] + tc[k][j]))
						tc[i][j] = tc[i][k] + tc[k][j]
						end
					end
				end
			end
		return tc
	end
	
	def printMatrix(m)
		print "ID   "
		m.size.times do |y|
			print "| " + (@nodeIds[y].to_s + "    ").slice(0,4) + " "
			end
		puts 
		print "-----"
		m.size.times do |y| print "|------"; end
		puts
					
		m.size.times do |y|
			print "" + (@nodeIds[y].to_s + "    ").slice(0,4) + " "
			m.size.times do |x|
				if (m[y][x]<9999)
					print "| " + ("    " + m[y][x].to_s).slice(-4,4) + " "
					else
					print "|    - "
					end
				end
			puts
			end
		
	end

end

