#
# Provides functions for analyzing pi-expressions in asynchronous pi-calculus
# Copyright (C) 2007 Frank Puhlmann.
#

require 'piexecutor.rb'


class AsyncPiAnalyzer

	attr_reader :agent
	attr_accessor :maxStates

	def initialize(agent,parser,sel)
		@agent = agent
		@parser = parser
		@sel = sel
		@maxStates = 100
		
		# Get id of End Event
	 	@endEventId = "N" + sel.to_s + "_" + parser.xml.elements.to_a("/model/process[@id='#{sel}']/node[@type='End Event']")[0].attributes['id']	 			
	 		 	
	end
	
	def remainingStates()	
		count = 0
		visited = Array.new
		
		@currentCount = 0
		@lastTransition = nil
		exec = AsyncPiExecutor.new(AgentRef.new(@agent,nil))
		
		@sound = true
		
		states = _remainingStates(exec,visited).to_s 
		
		puts "END EVENT ALWAYS LAST TRANSITION: " + @sound.to_s if states.to_i < @maxStates.to_i
		
		return states
		
	end
	
	
private

	def _remainingStates(exec,visited)
	
		# Check if current state has already been visited
		state = exec.currentAgent.to_definition
		if visited.include?(state)
			return 0
		end
		
		# Add to visited
		visited << state
		
		count = 0
	
		active_prefixes = exec.collectActivePrefixes	
		
		# Apply OR-join semantics 
		active_prefixes, blocked_prefixes = quick_or_join(active_prefixes,@parser,@sel)
		
		# Check if active prefix list is empty
		if active_prefixes.size==0
			# Test if last transitions was 'End Event'
			@sound = false if @lastTransition.to_s != @endEventId
			end

		# Execute them all within a separate copy
		active_prefixes.each do |p|
			# Safe last transition
			@lastTransition = ""
			case p
				when Node:
				@lastTransition = p.value.function if p.value.function
				end
			# Create copy
			execNew = AsyncPiExecutor.new(AgentRef.new(exec.currentAgent,nil),false)

			# Find matching prefixes in copy
			pnew = execNew.collectActivePrefixes
			pnew.each do |ptest|
				if ptest.to_s == p.to_s
					execNew.exec(ptest)
					break
				end
			end
			
			if @currentCount < @maxStates
				@currentCount += 1
				count = count + 1 + _remainingStates(execNew,visited)
				end

			end
		
		return count		
		
	end
	
end