require 'piexecutor'
require 'test/unit'

class TestAgent < Test::Unit::TestCase

	#
	# Test active prefixes (incl. intra-action)
	#
	def test_active_prefixes
		n = Agent.new("N", ['i','o'],  nil)
		n1 = Agent.new("N1", ['e1'], Tau.new(Tau.new(Output.new("e1"))))
		n2 = Agent.new("N2", ['e1','e2'], Output.new("e2"))
		n3 = Agent.new("N3", ['e2'],  Input.new("e2",Tau.new(Input.new("e1"))))
		n4 = Agent.new("N4", ['e1'], Input.new("e1",Tau.new))
		c = Composition.new(n1,n2,n3)
		n.behavior = Restriction.new("e1",Restriction.new("e2",Composition.new(c, n4)))
		
    	# Resolve everything into one executable agent!
    	exAgent = Agent.new("__EX", n.params, flatCopy(n.behavior))
    	
    	puts exAgent.to_definition
								
		active_nodes = collectActivePrefixes(exAgent.behavior, exAgent)
		
		assert_equal(2,active_nodes.flatten.size)
				
		active_nodes.flatten.each do |n|
		
			puts n.class.to_s
					
			case n
			
			when Node:
				assert_equal("t.t.'e1#0.0", n.value.to_s)
				
			when IntraactionNodes
			
				assert_equal("e2#0.0/'e2#0.t.e1#0.0", n.output.value.to_s + "/" + n.input.value.to_s)
			
			end
			
		end

	end
	
	#
	# Test active prefixes (incl.summations)
	#
	def test_active_prefixes2
		n = Agent.new("N", ['i','o'], nil)
		n1 = Agent.new("N1", ['e1'], Tau.new(Tau.new(Output.new("e1"))))
		n2 = Agent.new("N2", ['e1','e2'], Output.new("e2"))
		n3 = Agent.new("N3", ['e2'], Input.new("e2",Tau.new(Input.new("e1"))))
		n4 = Agent.new("N4", ['e1'], Input.new("e1",Tau.new))
		c = Summation.new(n1,n2,n3)
		n.behavior = Restriction.new("e1",Restriction.new("e2",Composition.new(c, n4)))
		
    	# Resolve everything into one executable agent!
    	exAgent = Agent.new("__EX", n.params, flatCopy(n.behavior))
				
		active_nodes = collectActivePrefixes(exAgent.behavior, exAgent)
		
		assert_equal(1,active_nodes.flatten.size)
		
		active_nodes.flatten.each do |n|
		
			case n
			
			when Node:
				assert_equal("t.t.'e1#1.0", n.value.to_s)
				
			when IntraactionNodes
				fail "No intra-action node expected!"
			
			end
			
		end

	end
	
	#
	# Test intra-actions via scoped names
	#
	def test_active_prefixes3
	
		n1 = Agent.new("N1",nil,Restriction.new("o",Composition.new(Output.new("o"),Input.new("o"))))
		n2 = Agent.new("N2",nil,Composition.new(Restriction.new("o",Output.new("o")),
		                                        Restriction.new("o",Input.new("o"))))
		
		exAgent1 = Agent.new("__EX1", n1.params, flatCopy(n1.behavior))
		exAgent2 = Agent.new("__EX2", n2.params, flatCopy(n2.behavior))

		active_nodes1 = collectActivePrefixes(exAgent1.behavior, exAgent1)	
		active_nodes2 = collectActivePrefixes(exAgent2.behavior, exAgent2)
		
		assert_equal(1, active_nodes1.size)
		assert_equal(IntraactionNodes, active_nodes1[0].class)
		
		assert_equal(0, active_nodes2.size)
		
	
	end


end